module chemloss_ch4_all
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform ch4 loss using pre-calculated fields from P. Bergamaschi (oh,cl)
  !
  !       interface_
  !       ---------
  !       call chemloss_apply_all
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_chemloss
  !-----------------------------------------------------------------------
  use chem_param ,         only: xmch4, xmc, ra
  use global_types,        only: d3_data
  use Dims,                only: nregions, nlon360,nlat180,lm
  use emission_data,       only: do_add_3d
  use chemloss_ch4_ohcl,   only: kohcl
  use ParTools,            only: maxtrac_per_pe
  use emission_common,     only: ch4_loss_m, ch4_loss_n, flux_means_are_priors, enkf_is_forward
  implicit none

  private

  ! public routines:
  public   :: chemloss_apply_ch4_all, declare_chemloss_ch4_all, free_chemloss_ch4_all, calc_chemloss_ch4_all
  public   :: pseudo

  type(d3_data),dimension(nregions),target                        :: ch4_loss

  logical                                                         :: loss_unallocated=.true.
  logical                                                         :: pseudo=.false.

contains

  subroutine declare_chemloss_ch4_all
   use dims,       only: im,jm,nregions
   use ParTools,   only: ntracetloc
   implicit none
   integer                                          :: region, imr, jmr, lmr, n

   do region=1,nregions
     imr=im(region)
     jmr=jm(region)
     lmr=lm(region)
     do n=1,ntracetloc
     allocate(ch4_loss(region)%d3(imr,jmr,lmr))
     end do
   enddo

  end subroutine declare_chemloss_ch4_all

  subroutine calc_chemloss_ch4_all
   use dims,       only: nlon360, nlat180, im,jm,lm, sec_month, newsrun, idate, newmonth, newday,nregions, itau, staggered, nread
   use ParTools,   only: tracer_active, ntracetloc
   use chem_param, only: xmch4
   use partools,   only : myid,root_t,nmembersloc
   implicit none
   integer                                          :: region, imr, jmr, lmr, n, mem
   integer, parameter                               :: add_field=0

   if (.not.newmonth) return  ! only every once per month

    do mem=1,nmembersloc

! create full emissions in mol/m2/s
    !if(myid==root_t) print*,itau,sum(ch4_loss(1)%d3(:,:,:)),mod(itau+staggered,nread/2),nread
      do region=1,nregions
!       ch4_loss(region)%d3(:,:,:)=kohcl(region)%d3(:,:,:)*sec_month !kgCH4/gridbox/s
       ch4_loss(region)%d3(:,:,:)=kohcl(region)%d3(:,:,:)
      enddo

       ! accumulate weekly average- at 60x45 resolution
       if(flux_means_are_priors .eq. .FALSE.) then
         if(allocated(ch4_loss_m)) then
          ch4_loss_m = ch4_loss_m+ch4_loss(1)%d3(:,:,:)
          ch4_loss_n=ch4_loss_n+1
         end if
       end if

 !      if(myid==root_t) print*,"DEBUG calc loss mem",mem,nmembersloc,sum(ch4_loss)


    end do

    ! accumulate weekly average
    if(flux_means_are_priors .eq. .TRUE.) then
       if(allocated(ch4_loss_m)) then
          ch4_loss_m=ch4_loss_m+ch4_loss(1)%d3(:,:,:)
          ch4_loss_n=ch4_loss_n+1
       end if
    end if


  end subroutine calc_chemloss_ch4_all

  subroutine chemloss_apply_ch4_all(region)
  !
    use dims,          only: okdebug
    use ParTools,      only: tracer_active, ntracetloc, myid, ntracet_ar

    implicit none
    ! io        
    integer, intent(in)        :: region
    ! local
    integer                    :: n, offsetn=0
    ! start
    if(myid > 0) offsetn=sum(ntracet_ar(0:myid-1))
    do n=1,ntracetloc
            call do_add_3d(region,offsetn+n,ch4_loss(region)%d3(:,:,:),ra(n),ra(n),-1.)
    enddo


    if(okdebug) write(*,*) 'end of chemloss_apply_ch4_all'

   end subroutine chemloss_apply_ch4_all

!==================================================================

   subroutine free_chemloss_ch4_all
   use dims,       only: nregions_max
   use ParTools,   only: ntracetloc
   implicit none
   integer                                          :: region, n

   do region=1,nregions
       deallocate(ch4_loss(region)%d3)
   enddo


   end subroutine free_chemloss_ch4_all

end module chemloss_ch4_all
