!
! Levels.
!

module dims_levels

  use const_ec_v, only : lme, a_ec, b_ec
  
  use dims_grid, only : nregions
  
  implicit none
  
  ! --- in/out ------------------------------
  
  public
  
  
  ! --- const -------------------------------
  
  
  integer, parameter ::  lm(nregions) = 91

  ! Only lowest layers are used for convec calculations (sub files)
  ! to avoid memory problems.
  ! Quote Michiel van Weele:
  !     "..., convectie kan wel tot ~70 hPa (19km) gaan in de tropen.
  !     Ik denk dat je de bovenste 33 lagen veilig kan vernullen."
  ! Thus, the lowest 91 - 33 = 58  should be enough.
  integer,parameter :: lmax_conv = 58

  ! select ECMWF half levels;
  ! TM levels are number bottom-up:
  integer, parameter   ::  echlevs(0:lm(1)) = (/ &
                                                             91, 90, &
                             89, 88, 87, 86, 85, 84, 83, 82, 81, 80, &
                             79, 78, 77, 76, 75, 74, 73, 72, 71, 70, &
                             69, 68, 67, 66, 65, 64, 63, 62, 61, 60, &
                             59, 58, 57, 56, 55, 54, 53, 52, 51, 50, &
                             49, 48, 47, 46, 45, 44, 43, 42, 41, 40, &
                             39, 38, 37, 36, 35, 34, 33, 32, 31, 30, &
                             29, 28, 27, 26, 25, 24, 23, 22, 21, 20, &
                             19, 18, 17, 16, 15, 14, 13, 12, 11, 10, &
                              9,  8,  7,  6,  5,  4,  3,  2,  1,  0 /)

  real, parameter  :: bt(1:lm(1)+1) = b_ec(echlevs)
  real, parameter  :: at(1:lm(1)+1) = a_ec(echlevs)


end module dims_levels
