! ----------------------------------------------------------------------
!
!   const_ec_v - vertical level definition of the ecwmf model
!
!   NOTE: this implementation defines the NCEP-RA 28 levels;
!     'ec' names are however maintained ...
!
! ----------------------------------------------------------------------

module const_ec_v

  implicit none

  ! --- in/out -----------------------------

  private

  public :: lme, a_ec, b_ec


  ! --- const --------------------------------

  ! number of levels:
  integer, parameter :: lme = 28
  
  !
  ! NCEP-RA is defined on full sigma levels:
  !
  !          0.9950, 0.9821, 0.9644, 0.9425, 0.9159, 0.8838, 0.8458, &
  !          0.8014, 0.7508, 0.6943, 0.6329, 0.5681, 0.5017, 0.4357, &
  !          0.3720, 0.3125, 0.2582, 0.2101, 0.1682, 0.1326, 0.1028, &
  !          0.0782, 0.058 , 0.0418, 0.0288, 0.0183, 0.0101, 0.0027
  !
  ! If the following half levels are used, the sigma values are more or less the averages,
  ! except for the last value, since the last half level sigma is set to zero 
  !
  !    1.0000  ,  0.9900  ,   0.9742  ,   0.9546 ,  &
  !    0.9304  ,  0.9014  ,   0.8662  ,   0.8254 ,  &
  !    0.7774  ,  0.7242  ,   0.6644  ,   0.6014 ,  &
  !    0.5348  ,  0.4686  ,   0.4028  ,   0.3412 ,  &
  !    0.2838  ,  0.2326  ,   0.1876  ,   0.1488 ,  &
  !    0.1164  ,  0.0892  ,   0.0672  ,   0.0488 ,  &
  !    0.0348  ,  0.0227  ,   0.0138  ,   0.0064 ,  &
  !    0.0000
  !


  ! Hybrid coordinate coefficients at half level interfaces.
  ! Coefficient a is given in [Pa] .
  ! Pressure at a certain hybrid level is defined as: p = a + b.SP

  real, parameter  :: a_ec(0:lme) = 0.0

  real, parameter  :: b_ec(0:lme) = (/   &
                       1.0000  ,  0.9900  ,   0.9742  ,   0.9546 ,  &
                       0.9304  ,  0.9014  ,   0.8662  ,   0.8254 ,  &
                       0.7774  ,  0.7242  ,   0.6644  ,   0.6014 ,  &
                       0.5348  ,  0.4686  ,   0.4028  ,   0.3412 ,  &
                       0.2838  ,  0.2326  ,   0.1876  ,   0.1488 ,  &
                       0.1164  ,  0.0892  ,   0.0672  ,   0.0488 ,  &
                       0.0348  ,  0.0228  ,   0.0138  ,   0.0064 ,  &
                       0.0000 /)

end module const_ec_v

